﻿(function (ng, $) {

    angular.module('forms')
    // data-validation-message
    .directive('validationMessage', ['$compile', function ($compile) {

        var getFieldValidationExpression = function (formName, fieldName) {
            var fieldExpression = formName + '.' + fieldName;
            var dirtyExpression = fieldExpression + '.$dirty';
            var invalidExpression = fieldExpression + '.$invalid';
            var watchExpression = invalidExpression + ' && ' + dirtyExpression;

            return watchExpression;
        };

        return {
            restrict: 'A',
            link: function (scope, element, attr) {
                var formName = element.parents("div[data-ng-form]").data("ng-form"); // "memForm";

                var fieldName = element.data("for"); // attr.for not working in IE8

                var watchExpression = getFieldValidationExpression(formName, fieldName);
                var template = '<div class="tooltip error ng-trans ng-trans-fade-right easeInBack" data-ng-if="' + watchExpression + '">' +
                                    '<span class="angle"></span>' +
                                    '<div class="tip"><p>' + attr.validationMessage + '</p></div>' +
                                '</div>' +
                                '<span class="tick ng-trans ng-trans-fade-right easeInBack" data-ng-if="' + formName + '.' + fieldName + '.$valid && ' + formName + '.' + fieldName + '.$dirty"></span>';

                element.html(template);
                $compile(element.contents())(scope);
            }
        };
    }])
    // data-validation-message
    .directive('errorGlow', ['$compile', '$window', function ($compile, $window) {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, element, attr, ngModel) {
                scope.$watch(function () { return ngModel.$invalid && ngModel.$dirty; }, function (invalid) {
                    invalid ? element.addClass('error') : element.removeClass('error');
                }, true);
            }
        };
    }])
    // data-unique-email
    .directive('uniqueEmail', [function ($timeout) {
        var toId;
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, elem, attr, ctrl) {
                //when the scope changes, check the email.
                scope.$watch(attr.ngModel, function (value) {
                    // if there was a previous attempt, stop it.
                    if (toId) {
                        clearTimeout(toId);
                    }

                    // http://stackoverflow.com/a/12865401/654708
                    // start a new attempt with a delay to keep it from getting too "chatty".
                    toId = $timeout(function () {
                        //                        // call to some API that returns { isValid: true } or { isValid: false }
                        //                        $http.get(attr.uniqueEmail + value).success(function (data) {
                        //
                        //                            //set the validity of the field
                        //                            $scope.$apply(function (s) {
                        //                                ctrl.$setValidity('uniqueEmail', data.isValid);
                        //                            });
                        //                        });
                        ctrl.$setValidity('uniqueEmail', true);
                    }, 200);
                });
            }
        };
    }])
    // data-email-check
    .directive('emailCheck', [function () {
        return {
            require: 'ngModel',
            link: function (scope, elem, attrs, ctrl) {
                var primaryEmail = "input[name=" + attrs.emailCheck + "]";
                elem.add(primaryEmail).on('keyup', function () {
                    scope.$apply(function () {
                        var v = elem.val() === $(primaryEmail).val();
                        ctrl.$setValidity('emailCheck', v);
                    });
                });
            }
        };
    }])
    // data-is-checked
    .directive('isChecked', [function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ctrl) {
                if (attrs.isChecked.length > 0) {
                    ctrl.$setViewValue(attrs.isChecked);
                    return;
                }

                ctrl.$setViewValue(attrs.value);
            }
        };
    }])
    // Marks an input as checked & updates it's ngModel 
    // w/ the value of it's "value" attribute when passed expression is true
    // data-check-when-true
    .directive('checkWhenTrue', [function () {
        return {
            require: 'ngModel',
            link: function (scope, elem, attrs, ctrl) {
                // Conditionally "checks" an input when passed expression is true
                scope.$watch(attrs.checkWhenTrue, function (boolExpression) {
                    //if (boolExpression && ctrl.$viewValue != attrs.value && !elem.is(":checked")) {
                    if (boolExpression) {
                        ctrl.$setViewValue(scope.$eval(attrs.value));
                        //ctrl.$modelValue = attrs.value;
                        //attrs.$set('ngModel', attrs.value);
                        elem.prop('checked', true);
                    }
                });
            }
        };
    }])
    // Works for checkboxes & radio buttons
    // data-when-checked
    .directive('whenChecked', [function () {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, elem, attrs, ctrl) {
                scope.$watch(attrs.ngModel, function (newVal) {
                    if (newVal && elem.is(":checked")) {
                        scope.$eval(attrs.whenChecked);
                    }
                });
            }
        };
    }])
    // Works for select elements
    // data-when-selected (used in conjunction with data-eval-exp-when-chosen)
    .directive('whenSelected', [function () {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, elem, attrs, ctrl) {
                scope.$watch(attrs.ngModel, function (newVal) {
                    scope.$eval(elem.find(":checked").data("evalExpWhenChosen"));
                });
            }
        };
    }]);
})(angular, window.jQuery || angular.element);
