﻿(function () {
    angular.module('mapModule')
        .controller('MapCtrl', ['$scope', '$http', 'GoogleMapService', function ($scope, $http, googleMapService) {
            $scope.googleMapObject = {}; // The Google map module. See googleMapModule.js
            $scope.searchResults = []; // Holds search results		
            $scope.searchPerformed = false; // Flag to track if the user has searched for anything yet

            $scope.selectOptions = {
                regionNames: [], // E.g. ["Auckland", "Wellington"];
                placesOfInterestTags: [], // E.g. ["Public Toilet", "Cafe"];
                allPlacesOfInterest: [],
                getPlacesNames: []
            };

            $scope.filter = {
                keyword: [],
                regionNames: [],
                placesOfInterestTags: [],
                placesOfInterestTagsFilter: "all" // Default to "all" so every facility chosen must exist in results
            };

            $scope.select2 = {
                keywordDefaults: {
                    multiple: true,
                    simple_tags: true,
                    maximumSelectionSize: 1,
                    tags: [],
                    formatSelectionTooBig: function (maxSize) {
                        return "Only " + maxSize + " search term is allowed";
                    },
                    formatResult: function (result, container, query, escapeMarkup) {
                        container.addClass('needsclick');
                        return result.text;
                    }
                },
                defaults: {
                    maximumSelectionSize: 10,
                    formatResult: function (result, container, query, escapeMarkup) {
                        container.addClass('needsclick');
                        return result.text;
                    }
                }
            };

            $scope.updateMap = function () {
                // Serialized the map filter values that a user has entered, into JSON & post it to our Google Map Web API
                var mapFilterValuesAsJson = angular.toJson({
                    Keyword: angular.isDefined($scope.filter.keyword) ? $scope.filter.keyword[0] : "",
                    RegionNames: angular.isDefined($scope.filter.regionNames) ? $scope.filter.regionNames : [],
                    PlacesOfInterestTags: angular.isDefined($scope.filter.placesOfInterestTags) ? $scope.filter.placesOfInterestTags : [],
                    MatchAllPlacesOfInterestTags: $scope.filter.placesOfInterestTagsFilter === "all" ? true : false
                });

                googleMapService.getPlaces(mapFilterValuesAsJson).then(function (results) {
                    $scope.searchResults = results;
                    $scope.numOfResults = $scope.searchResults.length;

                    if ($scope.numOfResults > 0) {
                        // Update map with results from Web API request
                        $scope.googleMapObject.addNewLocationMarkers($scope.searchResults);
                    }

                    if ($scope.searchPerformed === false) {
                        $scope.searchPerformed = true;
                    }
                });
            };

            populateSelectOptionsAsync();

            function populateSelectOptionsAsync() {
                googleMapService.getRegionNames().then(function (data) {
                    $scope.selectOptions.regionNames = data;
                });

                googleMapService.getTags().then(function (data) {
                    $scope.selectOptions.placesOfInterestTags = data;
                });

                googleMapService.getAllPlaces().then(function (data) {
                    $scope.selectOptions.allPlacesOfInterest = data;
                });

                googleMapService.getPlacesNames().then(function (data) {
                    angular.extend($scope.select2.keywordDefaults.tags, data);
                });
            }
        }]);
}());