﻿(function () {
    angular.module('mobileNav')
        .controller('NavCtrl', ['$scope', '$http', 'NavigationService', function ($scope, $http, navigationService) {

            $scope.navigation = navigationService;
            $scope.fullTree = [];
            $scope.flattenedTree = [];
            $scope.search = "";

            // Check local storage first
            if (navigationService.navInStorage()) {
                $scope.fullTree = navigationService.getNavFromStorage();
                // console.info("Mobile navigation pulled from " + navigationService.storageMedium + ".");
            } else {
                // Call the async method and then do stuff with what is returned inside the function
                navigationService.getNavAsync()
                .then(function (asyncData) {
                    // console.info("Mobile navigation pulled asyncronously.");
                    $scope.fullTree = asyncData;
                });
            }

            // Watch the fullTree nav array for any changes.
            // When the array gets populated with values, build the flattened array for search
            $scope.$watchCollection('fullTree', function (nav) {
                if (nav.length > 0) {
                    flattenJsonArray($scope.fullTree, $scope.flattenedTree, "Children", ["Children"]);
                    //console.log("Nav tree is flattened");
                }
            });

            // Flattens an array using recursion.
            // recursivePropertyName = the name of the property to call the recursion on e.g. "children"
            // propertiesToRemoveArray = an array of properties to exclude from the flattened array e.g. ["children", "url"]
            function flattenJsonArray(originalArray, flattenedArray, recursivePropertyName, propertiesToRemoveArray) {
                if (!originalArray) return;

                var len = originalArray.length, i = 0; // cache array size for better speed
                for (i; i < len; i++) {
                    var item = originalArray[i];
                    var newItem = angular.extend({}, item);
                    if (propertiesToRemoveArray) {
                        var innerLen = propertiesToRemoveArray.length, j = 0;
                        for (j; j < innerLen; j++) {
                            //delete newItem[propertiesToRemoveArray[j]]; // slower
                            newItem[propertiesToRemoveArray[j]] = null;
                        }
                    }
                    flattenedArray.push(newItem);
                    flattenJsonArray(item[recursivePropertyName], flattenedArray, recursivePropertyName, propertiesToRemoveArray);
                }
            }
        }]);
}());