﻿/*
    Mobile Nav directives
*/

(function (ng, $) {

    angular.module('mobileNav')
    // data-add-mobile-nav-transition 
    .directive('addMobileNavTransition', ['$timeout', function ($timeout) {

        var navigateMenuToActivePage = function($menu) {
            var pathArray = window.location.pathname.split('/'),
                arrLength = pathArray.length,
                pathBuilder = "";

            for (var i = 0; i < arrLength; i++) {
                var path = "/" + pathArray[i];
                if (path.length < 2 || i === (arrLength - 1)) {
                    continue;
                }

                pathBuilder += path;

                //console.log(pathBuilder);
                $menu.find('a[href^="' + pathBuilder + '"]')
                    .first()
                    .next('.showChildren')
                    .trigger('click');
            }
        };

        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                var slideAmount = 0;

                // Using jQuery "on" here as HTML for the mobile nav has not been added 
                // to the DOM by the time this directive runs
                element.on('click', 'li.hasList > .showChildren', function (e) {
                    $(this).nextAll("ul").addClass("active");
                    slideAmount = slideAmount - 100;
                    // jQuery will vendor-prefix "tranform" e.g. "-webkit-transform: translate3d(0, 0, 0);" for Chrome
                    element.css({ "transform": "translate3d(" + slideAmount + "%,0,0)" });
                });

                element.on('click', '.backLink', function (e) {
                    // cache selector for use in setTimeout
                    var $parent = $(this).parent("ul");
                    slideAmount = slideAmount + 100;
                    // jQuery will vendor-prefix "tranform" e.g. "-webkit-transform: translate3d(0, 0, 0);" for Chrome
                    element.css({ "transform": "translate3d(" + slideAmount + "%,0,0)" });
                    // Wait 300ms for the transform to complete
                    setTimeout(function () {
                        $parent.removeClass("active");
                    }, 300);
                });
                
                $(function () {
                    setTimeout(function () {
                        navigateMenuToActivePage(element);
                    }, 1000);
                });
            }
        };
    }])
    // data-overlay-toggle
    .directive('overlayToggle', ['$timeout', '$window', function ($timeout, $window) {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                scope.newFastClickAdded = false;

                var $overlay = element;
                attrs.$observe('overlayToggle', function (value) {
                    if (value == 'true') {
                        // Reinitialise Fastclick for dynamicaly added menu links
                        if (!scope.newFastClickAdded && $window.FastClick) {
                            new FastClick.attach(document.body);
                            scope.newFastClickAdded = true;
                        }

                        $('body').css('overflow-x', 'hidden');
                        $overlay.show(0, function () {
                            $overlay.addClass('show');
                        });
                    } else {
                        $overlay.removeClass('show');
                        $timeout(function () {
                            $overlay.hide();
                            $('body').css('overflow-x', '');
                        }, 200);
                    }
                });
            }
        };
    }]);
})(angular, window.jQuery || angular.element);