﻿(function (ng) {
    angular.module('search')
        .factory('RegisterSearchService', ['$http', function ($http) {

            var registerApiUrl = "/api/register/GetRegisterSearchResults",
            registerCsvApiUrl = "/api/register/GetRegisterResultsCsv",
            registerApiEnableCsvParams = "&format=csv",
            fullRegisterCsvExportUrl = registerCsvApiUrl + "/?format=csv";
            
            var SearchData = function () {
                var that = {};
                that.address = '';
                that.search = [];
                that.chooseARegion = [];
                that.registrationType = [];
                that.uses = [];
                that.theme = '';
                that.constructionProfessional = '';
                that.localAuthority = [];
                that.searchAll = '';
                that.lat = 0;
                that.lng = 0;
                that.userLocationLat = 0;
                that.userLocationLng = 0;
                that.pageNumber = 1;
                that.pageSize = 20;
                that.requestedUrl = '';
                that.searchPassed = false;
                that.searchedTerm = '';
                that.googleAutoCompleteResults = {};
                that.searchResultsCount = 0;
                that.fullRegisterCsvExportUrl = fullRegisterCsvExportUrl;
                that.pagedRegisterCsvExportUrl = "";
                that.townCity = [];
                that.place = [];
                
                that.validSearchData = function () {
                    return ((!this.lat || this.lat == 0) &&
                        (!this.lng || this.lng == 0) &&
                        (!this.search || !this.search.length) &&
                        (!this.chooseARegion || !this.chooseARegion.length) &&
                        (!this.registrationType || !this.registrationType.length) &&
                        (!this.localAuthority || !this.localAuthority.length) &&
                        (!this.uses || !this.uses.length) &&
                        (!this.theme || !this.theme.length) &&
                        (!this.constructionProfessional || !this.constructionProfessional.length) &&
                        (!this.searchAll || !this.searchAll.length)) &&
                        (!this.townCity || !this.townCity.length) &&
                        (!this.place || !this.place.length)
                        ? false
                        : true;
                };

                return that;
            };

            var Select2Options = function () {
                var select2 = {
                    options: {
                        multiple: true,
                        simple_tags: true,
                        maximumSelectionSize: 1,
                        minimumInputLength: 1,
                        separator: "|",
                        ajax: {
                            quietMillis: 500,
                            url: "/api/register/GetTypeaheadValues",
                            data: function (term, page) {
                                return {
                                    value: term
                                }; // query params go here
                            },
                            results: function (data, page) { // parse the results into the format expected by Select2.
                                // since we are using custom formatting functions we do not need to alter remote JSON data
                                return { results: data };
                            }
                        },
                        formatResult: function (result, container, query, escapeMarkup) {
                            container.addClass('needsclick');
                            return result.text;
                        },
                        //Allow manually entered text in drop down.
                            createSearchChoice: function (term, data) {
                                if ($(data).filter(function () {
                                  return this.text.localeCompare(term) === 0;
                                }).length === 0) {
                                    return { id: term, text: term };
                                }
                            }
                    },
                    listNumbers: {
                        multiple: true,
                        simple_tags: true,
                        maximumSelectionSize: 1,
                        minimumInputLength: 1,
                        separator: "|",
                        ajax: {
                            quietMillis: 500,
                            url: "/api/register/GetListNzaaNumbers",
                            data: function (term, page) {
                                return {
                                    value: term
                                }; // query params go here
                            },
                            results: function (data, page) { // parse the results into the format expected by Select2.
                                // since we are using custom formatting functions we do not need to alter remote JSON data
                                return { results: data };
                            }
                        },
                        formatResult: function (result, container, query, escapeMarkup) {
                            container.addClass('needsclick');
                            return result.text;
                        },

                    },
                    places: {
                        multiple: true,
                        simple_tags: true,
                        maximumSelectionSize: 1,
                        minimumInputLength: 1,
                        separator: "|",
                        ajax: {
                            quietMillis: 500,
                            url: "/api/register/GetPlaceNames",
                            data: function (term, page) {
                                return {
                                    value: term
                                }; // query params go here
                            },
                            results: function (data, page) { // parse the results into the format expected by Select2.
                                // since we are using custom formatting functions we do not need to alter remote JSON data
                                return { results: data };
                            }
                        },
                        formatResult: function (result, container, query, escapeMarkup) {
                            container.addClass('needsclick');
                            return result.text;
                        },
                        //Allow manually entered text in drop down.
                        createSearchChoice: function (term, data) {
                            if ($(data).filter(function () {
                              return this.text.localeCompare(term) === 0;
                            }).length === 0) {
                                return { id: term, text: term };
                            }
                        }
                    },
                    townCity: {
                        multiple: true,
                        simple_tags: true,
                        maximumSelectionSize: 5,
                        minimumInputLength: 1,
                        separator: "|",
                        formatResult: function (result, container, query, escapeMarkup) {
                            container.addClass('needsclick');
                            return result.text;
                        },
                    },
                    defaults: {
                        maximumSelectionSize: 5,
                        formatResult: function (result, container, query, escapeMarkup) {
                            container.addClass('needsclick');
                            return result.text;
                        }
                    }
                };

                return select2;
            };

            var RegisterSearch = function (searchData) {
                angular.extend(this, searchData);

                this.totalResultsCount = 0;
                this.allResults = [];
                this.pagedResults = [];
                this.pagedRegisterCsvExportUrl = "";
                this.busy = false;

                this.initialSearchHasRun = false;
                this.allItemsLoaded = function () {
                    return (this.pagedResults.length >= this.totalResultsCount);
                };
                this.isAddressSearch = function () {
                    return this.address;
                };
            };

            var RegisterSearchDto = function (registerSearch) {
                this.keyword = registerSearch.search;
                this.searchAll = registerSearch.searchAll;
                this.address = registerSearch.address;
                this.lat = registerSearch.lat;
                this.lng = registerSearch.lng;
                this.userLocationLat = registerSearch.userLocationLat;
                this.userLocationLng = registerSearch.userLocationLng;
                this.regions = registerSearch.chooseARegion;
                this.registrationType = registerSearch.registrationType;
                this.uses = registerSearch.uses;
                this.theme = registerSearch.theme;
                this.constructionProfessional = registerSearch.constructionProfessional;
                this.localAuthorities = registerSearch.localAuthority;
                this.pageNumber = registerSearch.pageNumber;
                this.pageSize = registerSearch.pageSize;
                this.includeAllResults = false;
                this.townCity = registerSearch.townCity;
                this.place = registerSearch.place;
            };

            var buildUrl = function (baseUrl, object) {
                return baseUrl + "?" + $.param(object, true);
            };

            RegisterSearch.prototype.getAllRegisterSearchResults = function () {
                this.busy = true;

                var searchDto = new RegisterSearchDto(this);
                searchDto.includeAllResults = true;

                // $http returns a promise, which has a then function, which also returns a promise
                return $http({ cache: false, dataType: 'json', url: registerApiUrl, method: 'GET', params: searchDto })
                    .then(function (response) {

                        searchDto.includeAllResults = false;
                        this.totalResultsCount = response.data.Count;
                        this.pagedRegisterCsvExportUrl = buildUrl(registerCsvApiUrl, searchDto) + registerApiEnableCsvParams;

                        searchDto.pageSize = this.totalResultsCount;
                        this.allRegisterResultsCsvExportUrl = buildUrl(registerCsvApiUrl, searchDto) + registerApiEnableCsvParams;

                        this.allResults = response.data.Results;

                        if (this.isAddressSearch()) {
                            if(this.allResults[0] != null)
                                this.allResults[0].zIndex = 999998;
                            if (this.allResults[1] != null)
                                this.allResults[1].zIndex = 999999;
                            
                            this.pagedResults = response.data.Results.slice(1, this.pageSize + 1);
                        } else {
                            this.pagedResults = response.data.Results.slice(0, this.pageSize);
                        }

                        this.pageNumber++;
                        this.initialSearchHasRun = true;
                        this.busy = false;
                    }.bind(this));
            };

            RegisterSearch.prototype.nextPage = function () {
                if (this.busy || this.allItemsLoaded()) {
                    return;
                }

                this.busy = true;

                // Push next results to array
                var startIndex = (this.pageNumber - 1) * this.pageSize;
                if (this.isAddressSearch()) {
                    startIndex = startIndex + 1;
                }

                var resultsForThisPage = this.allResults.slice(startIndex, startIndex + this.pageSize);
                this.pagedResults.push.apply(this.pagedResults, resultsForThisPage);

                // Build register CSV export url
                var searchDto = new RegisterSearchDto(this);
                var searchDtoClone = angular.copy(searchDto);
                searchDtoClone.pageSize = searchDtoClone.pageSize * searchDtoClone.pageNumber;
                searchDtoClone.pageNumber = 1;
                //this.pagedRegisterCsvExportUrl = buildUrl(registerApiUrl, searchDtoClone);

                this.pageNumber++;
                this.busy = false;
            };

            return {
                getInstance: function (searchData) {
                    return new RegisterSearch(searchData);
                },
                getRegions: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetRegions', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getGroupedUses: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetGroupedUses', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getUses: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetUses', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getLocalAuthority: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetLocalAuthority', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getLocalAuthorityByRegion: function (regions) {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: false, dataType: 'json', url: '/api/register/GetLocalAuthorityByRegion', method: 'GET', params: { regions: regions } })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getRegistrationType: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetRegistrationType', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                getTypeaheadValues: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetTypeaheadValues', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },
                
                getPlaceNames: function () {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: true, dataType: 'json', url: '/api/register/GetPlaceNames', method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },

                getFilteredTypeaheadValues: function (value) {
                    // $http returns a promise, which has a then function, which also returns a promise
                    return $http({ cache: false, dataType: 'json', url: '/api/register/GetTypeaheadValues?value=' + value, method: 'GET' })
                                .then(function (response) {
                                    return response.data;
                                });
                },

                fullRegisterCsvExportUrl: fullRegisterCsvExportUrl,

                searchData: SearchData,

                select2Options: Select2Options
            };

        }]);
})(angular);