﻿/*
    Reusable directive
*/

(function (ng, $) {
    angular.module('reusableDirectives')
    // data-on-click-redirect-to
    .directive('onClickRedirectTo', ['$window', function ($window) {
        return {
            restrict: 'A',
            link: function (scope, element, attr) {
                element.click(function (e) {
                    e.preventDefault();
                    $window.location.href = attr.onClickRedirectTo;
                });
            }
        };
    }])
    // data-slide-toggle
    .directive('slideToggle', [function () {
        return {
            restrict: 'A',
            scope: {},
            link: function (scope, element, attr) {
                element.css('cursor', 'pointer').click(function () {
                    var speed = attr.slideToggleSpeed ? parseInt(attr.slideToggleSpeed, 10) : 200;
                    $(attr.slideToggle).slideToggle(speed);
                });
            }
        };
    }])
    // data-search-reset
    .directive('searchReset', [function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ngModel) {
                var $icon = $('<span class="icon search"></span>');
                elm.wrap('<span class="icon-wrap" />').after($icon);

                $icon.bind('click', function (e) {
                    scope.$apply(function () {
                        // if data-search-reset has a value use it as the default value for the text box when clearing
                        if (attrs.searchReset) {
                            ngModel.$setViewValue(scope.$eval(attrs.searchReset));
                        } else {
                            ngModel.$setViewValue();
                        }
                        ngModel.$render();
                    });
                });

                elm.bind('change keyup mouseover', function () {
                    if (elm.val()) {
                        $(this).parent().find('.search').toggleClass('search clear');
                    }
                });

                elm.parent()
                .bind('mouseleave focusout', function () {
                    if (!elm.val()) {
                        $(this).find('.clear').toggleClass('clear search');
                    }
                });

                setTimeout(function () {
                    if (elm.val()) {
                        elm.trigger('mouseover');
                    }
                }, 500);
            }
        };
    }])
    // data-fill-viewport-height
    .directive('fillViewportHeight', ['$window', function ($window) {
        return {
            restrict: 'A',
            link: function (scope, element, attr) {
                var widowResized;

                $(document).ready(function () {
                    fillViewport();
                });

                $(window).resize(function () {
                    clearTimeout(widowResized);
                    widowResized = setTimeout(fillViewport, 100);
                });

                function fillViewport() {
                    var offset = 0;
                    $(attr.fillViewportHeight).each(function (i) {
                        offset += $(this).outerHeight(true);
                    });

                    var viewportHeight = $(window).height();
                    element.height(viewportHeight - (offset + 1)); // +1 for round-off error
                    //alert(viewportHeight - (offset + 1));
                }
            }
        };
    }])
    // data-match-height-of
    .directive('matchHeightOf', ['$parse', function ($parse) {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                var matchHeightTimer;
                var matchHeightOfOptions = $parse('{' + attrs.matchHeightOf + '}')(scope);

                // If no valid selector passed then exit
                if (!matchHeightOfOptions.selector || !matchHeightOfOptions.selector.length) {
                    return;
                }

                var matchHeightOnWindowResize = matchHeightOfOptions.matchHeightOnWindowResize;
                var $ele = $(matchHeightOfOptions.selector).first();

                function matchHeight() {
                    var heightToSet = $ele.outerHeight(true);
                    element.height(parseInt(heightToSet, 10));
                }

                matchHeightTimer = setTimeout(matchHeight, 500);

                if (matchHeightOnWindowResize) {
                    $(window).resize(function () {
                        clearTimeout(matchHeightTimer);
                        matchHeightTimer = setTimeout(matchHeight, 500);
                    });
                }
            }
        };
    }])
    // data-hide-empty-elements
    .directive('hideEmptyElements', [function () {
        return {
            restrict: 'A',
            link: function (scope, element, attr) {
                var tagsToCheck = attr.hideEmptyElements.length ? attr.hideEmptyElements : "*";
                tagsToCheck.replace(/,/g, ":empty,");
                element.find(tagsToCheck + ':empty').hide();
            }
        };
    }])
    // data-jq-flexslider - http://www.woothemes.com/flexslider/
    .directive('jqFlexslider', [function () {
        return {
            link: function (scope, element, attrs) {
                var sliderPrevNextNav = function (slider) {
                    $(attrs.jqFlexsliderNextBtn).click(function (event) {
                        slider.flexAnimate(slider.getTarget("next"));
                        slider.pause();
                    });
                    $(attrs.jqFlexsliderPrevBtn).click(function (event) {
                        slider.flexAnimate(slider.getTarget("prev"));
                        slider.pause();
                    });
                };

                var flexsliderOptions = scope.$eval('{' + attrs.jqFlexslider + '}');
                flexsliderOptions.start = sliderPrevNextNav;

                $(window).load(function () {
                    element.flexslider(flexsliderOptions);
                });
            }
        };
    }])
    // class="pie"
    .directive('pie', [function () {
        return {
            restrict: 'AC',
            link: function (scope, element, attrs) {
                $(function () {
                    if (window.PIE) {
                        window.PIE.attach(element[0]);
                    }
                });
            }
        };
    }])
    // data-no-paste
    .directive('noPaste', [function () {
        return {
            link: function (scope, element, attrs) {
                if (element.is('input')) {
                    element.bind('paste', function (e) {
                        e.preventDefault();
                    });
                }
            }
        };
    }])
    // data-google-address-autocomplete
    .directive('googleAddressAutocomplete', ['$window', '$timeout', '$parse', 'utils', function ($window, $timeout, $parse, utils) {
        return {
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {
                
                element.css({ "min-height": "30px" });

                var initAddressAutoComplete = function () {

                    var model = $parse(attrs.googleAddressAutocomplete);
                    var autocomplete = new $window.google.maps.places.Autocomplete(element[0], { componentRestrictions: { country: 'nz' } });

                    // Disable the enter button from submitting the form so AJAX call has a chance to finish
                    element.bind("keypress keyup", function (event) {
                        if (event.which === 13) {
                            event.stopPropagation(); // stop parent propagation
                        }
                    });
                    
                    // FastClick Fix
                    $(document).on({
                        'DOMNodeInserted': function () {
                            $('.pac-item, .pac-item span', this).addClass('needsclick');
                        }
                    }, '.pac-container');

                    $window.google.maps.event.addListener(autocomplete, 'place_changed', function () {
                        scope.$apply(function () {
                            var location = {
                                address: autocomplete.getPlace().formatted_address,
                                lat: autocomplete.getPlace().geometry.location.lat(),
                                lng: autocomplete.getPlace().geometry.location.lng()
                            };

                            //console.log("setting address obj", location);
                            model.assign(scope, location);
                            ngModel.$setViewValue(location.address);
                            ngModel.$render();
                        });
                    });
                };

                // Poll for the places library
                utils.poll(function () {
                    return 'google' in $window && 'places' in $window.google.maps;
                }, initAddressAutoComplete, function () {
                    throw "Timeout reached - Places library was not loaded for Google Maps. Address autocomplete will not work.";
                }, null, 5000);

            }
        };
    }])
    // data-clear-and-disable
    .directive('clearAndDisable', [function () {
        return {
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {
                attrs.$observe('clearAndDisable', function (modelValue) {
                    if (modelValue === 'true' || modelValue.length) {
                        ngModel.$setViewValue();
                        element.prop('disabled', true);
                        if (element.select2) {
                            element.select2("enable", false);
                        }
                    } else {
                        element.prop('disabled', false);
                        if (element.select2) {
                            element.select2("enable", true);
                        }
                    }
                    ngModel.$render();
                });
            }
        };
    }])
    // data-legacy-placeholder-pollyfill
    .directive('legacyPlaceholderPollyfill', [function () {
        return {
            link: function (scope, element, attrs) {
                element.find('input, textarea').placeholder();
            }
        };
    }])
    // data-submit-form-on-click
    .directive('submitFormOnClick', [function () {
        return {
            link: function (scope, element, attrs) {
                var $closestForm = element.closest('form');
                element.on('click', function () {
                    var searchVal = attrs.submitFormOnClick;
                    if (searchVal) {
                        $closestForm.submit();
                    }
                });
            }
        };
    }])
    // data-on-enter-key
    .directive('onEnterKey', [function () {
        return function (scope, element, attrs) {
            element.bind("keypress keyup", function (event) {
                if (event.which === 13) {
                    scope.$apply(function () {
                        scope.$eval(attrs.onEnterKey, { 'event': event });
                    });
                    event.preventDefault();
                }
            });
        };
    }])
    // data-only-digits
    .directive('onlyDigits', [function () {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                element.keydown(function (e) {
                    var key = e.which || e.keyCode;
                    if (//!e.shiftKey && !e.altKey && !e.ctrlKey &&
                        // numbers   
                        key >= 48 && key <= 57 ||
                        // Numeric keypad
                        key >= 96 && key <= 105 ||
                        // comma, period and minus, . on keypad
                        // key == 190 || key == 188 || key == 109 || key == 110 ||
                        // Backspace and Tab and Enter
                       key == 8 || key == 9 || key == 13 ||
                        // Home and End
                       key == 35 || key == 36 ||
                        // left and right arrows
                       key == 37 || key == 39 ||
                        // Del and Ins
                       key == 46 || key == 45)
                        return true;

                    return false;
                });
            }
        };
    }])
    // data-on-resize
    .directive('onResize', [function () {
        return function (scope, element, attrs) {
            var resizeTriggered;

            element.bind("resize", function (event) {
                // Disabled as overrides google map resize listener
                clearTimeout(resizeTriggered);
                resizeTriggered = setTimeout(function () {
                    eventCallback(event);
                }, 300);
            });

            function eventCallback(event) {
                scope.$apply(function () {
                    scope.$eval(attrs.onResize, {
                        'event': event
                    });
                });
                event.preventDefault();
            }
        };
    }])
    // data-loading-icon
    .directive('loadingIcon', ['$window', '$compile', function ($window, $compile) {
        var modernBrowserTemplate = '<div class="windows8 {{ loadingIcon && loadingIcon }}"> <div class="wBall" id="wBall_1"> <div class="wInnerBall"> </div> </div> <div class="wBall" id="wBall_2"> <div class="wInnerBall"> </div> </div> <div class="wBall" id="wBall_3"> <div class="wInnerBall"> </div> </div> <div class="wBall" id="wBall_4"> <div class="wInnerBall"> </div> </div> <div class="wBall" id="wBall_5"> <div class="wInnerBall"> </div> </div> </div>';
        var legacyBrowserTemplate = '<img data-ng-src="{{ legacyLoadingPath }}" alt="Animation icon for loading" />';

        var getTemplate = function (isModernBrowser) {
            return isModernBrowser ? modernBrowserTemplate : legacyBrowserTemplate;
        };

        var linker = function (scope, element, attrs) {
            // Requires Modernizr
            var isModernBrowser = angular.isDefined($window.Modernizr) && $('html').hasClass('csstransforms') && $('html').hasClass('cssanimations');
            var compiledTemplate = $compile(getTemplate(isModernBrowser))(scope);
            element.replaceWith(compiledTemplate);
        };

        return {
            restrict: "A",
            link: linker,
            replace: true,
            scope: {
                legacyLoadingPath: '@',
                loadingIcon: '@'
            }
        };
    }])
    // data-scroll-to-top
    .directive('scrollToTop', [function () {
        return {
            link: function (scope, element, attrs) {
                attrs.$observe('scrollToTop', function (observedValue) {
                    if (observedValue == "true") {
                        $("html, body").animate({ scrollTop: 0 }, 0);
                    }
                });
            }
        };
    }])
    // data-if-broken-image
    .directive('ifBrokenImage', [function () {
        return {
            link: function (scope, element, attrs) {
                var brokenImageObj = scope.$eval('{' + attrs.ifBrokenImage + '}');
                element.bind('error', function () {
                    if (brokenImageObj.remove) {
                        element.remove();
                        return;
                    }
                    element.attr('src', brokenImageObj.fallbackImagePath);
                });
            }
        };
    }])
    // data-set-form-action-as
    .directive('setFormActionAs', [function () {
        return {
            link: function (scope, element, attrs) {
                var $form = element.closest('form');
                $form.attr('action', attrs.setFormActionAs);
            }
        };
    }]);

})(angular, window.jQuery || angular.element);
