﻿(function () {
    angular.module('search')
        .controller('SearchCtrl', ['$scope', '$http', 'SearchService', '$filter', '$window', function ($scope, $http, SearchService, $filter, $window) {
            $scope.filter = {
                includeDocuments: false,
                pageSize: 10,
                dateFrom: null,
                dateTo: null
            };

            $scope.displayTooltip = function (prop) {
                $scope[prop] = !$scope[prop];

                if ($scope[prop]) {
                    $window.onclick = function (event) {
                        if (event.target.id != prop) {
                            $scope.$apply(function () {
                                $scope[prop] = false;
                            });
                        }
                    };
                }
            };
            
            $scope.siteSections = [];
            $scope.newsAndEventsIds = [];
            $scope.searchService = {};
            $scope.searchedTerm = "";
            $scope.searchResults = [];
            $scope.searchPerformed = false;
            $scope.datepickerDisabled = true;

            $scope.pauseSearch = function () {
                return !!($scope.searchPerformed === false || $scope.searchService.busy);
            };

            $scope.hasSearchResults = function () {
                return $scope.searchPerformed && !$scope.loadingAsyncResults() && $scope.searchService.results.length > 0;
            };

            $scope.loadingAsyncResults = function () {
                return $scope.searchService.busy;
            };

            $scope.noResults = function () {
                return $scope.searchPerformed && $scope.searchService.results.length === 0 && !$scope.loadingAsyncResults();
            };

            $scope.getSearchResults = function () {
                $scope.filter.searchedTerm = $scope.filter.search;
                $scope.searchService = new SearchService.GetInstance($scope.filter);
                $scope.searchService.nextPage(); // 1st set of results
                $scope.searchPerformed = true;
            };

            $scope.$watch('filter.searchPassed', function (val) {
                if (val === true) {
                    $scope.getSearchResults();
                }
            });

            // Enable Datepicker when news or events item is selected
            $scope.$watch('filter.siteSections', function (siteSections) {
                if (siteSections && (siteSections.length <= $scope.newsAndEventsIds.length)) {

                    var shouldEnableDatepicker = false;

                    angular.forEach($scope.newsAndEventsIds, function (newsAndEventsId) {
                        if (siteSections.indexOf(newsAndEventsId) > -1) {
                            shouldEnableDatepicker = true;
                        }
                    });

                    $scope.datepickerDisabled = !shouldEnableDatepicker;
                } else {
                    $scope.datepickerDisabled = true;
                    $scope.filter.dateFrom = "";
                    $scope.filter.dateTo = "";
                }
            });


            getNavSections();

            // Private function to populate site-sections drop down
            function getNavSections() {
                $http.get("/api/cms/GetNavSections")
                    .success(function (data, status) {
                        $scope.siteSections = data;
                        $scope.newsAndEventsIds = findNewsOrEventItems(data);
                    })
                    .error(function (data, status) {
                        $scope.siteSections = [];
                        // console.error("Could not get nav sections from service:", status);
                    });

                function findNewsOrEventItems(navObjectArray) {
                    var newsAndEventIds = [];

                    var newsAndEventObjects = $filter('filter')(navObjectArray, function (item) {
                        return (item.Heading).toLowerCase().indexOf('news') >= 0 || (item.Heading).toLowerCase().indexOf('event') >= 0;
                    });

                    angular.forEach(newsAndEventObjects, function (item) {
                        this.push(item.TargetId);
                    }, newsAndEventIds);

                    return newsAndEventIds;
                };
            };

        }]);
}());